function result=pf_colortracker(videofile, gtruth, outfile, SHOW_DATA)
% PF_COLORTRACKER performs single-object tracking across a video sequence.
%   Target initialization is obtained as the first entry in the
%   ground-truth file. The analysis is performed until the last frame
%   annotated in the ground-truth file.
%
%   This implementation is based on the color-based particle filter available
%   at http://www.mathworks.com/matlabcentral/fileexchange/17960
%
%   INPUT:
%       - videofile: path of the video sequence to analyze
%       - gtruth: path of the associated ground-truth
%       - outfile: filename of the MAT file with results
%       - show_data: flag to display (=1) or not (=0) (default 0)
%   OUTPUT:
%       - A figure (*_FWD.fig) is saved comparing the results of the
%       ground-truth error, the observation likelihood statistica and the
%       spatial uncertainty of particles statistic. 
%       - Structure 'result' with the following fields:
%           -'videofile': path of the input videofile
%           -'gtruthfile': path of the associated ground-truth
%           -'start_frame': start frame of the analyzed frames
%           -'end_frame': end frame o of the analyzed frames
%           -'N': number of samples (particles) used in the tracker
%           -'Q': target model used during the tracking process
%           -'Ck': noise of the motion model used in the prediction phase
%           -'w_par_N': weights of the particles
%           -'w_par_NN': particles likelihood (no normalized weights)
%           -'state_par': state of particles
%           -'state_est': state estimation for the target
%           -'der': ground-truth error (ellipse overlap)
%           -'OL': Observation likelihood statistic of the particles
%           -'SU': Spatial uncertainty statistic of the particles
%
%   If SHOW_DATA flag is activated, data displayed correspond to:
%      - Green ellipses: Ground-truth data
%      - Red ellipse: tracker estimation
%      - Blue crosses: tracker particles
%
%   Configuration of the color-based particle filter:
%
%    - State definition 
%
%    S_k = (x_k , y_k , H_k^x , H_k^y , theta_k)
%
%    - System dynamics
%
%           S_k = A_k S_{k-1} + N(0 , R_k)
%
%          |1         0       0        0       0      0     0|
%          |0         1       0        0       0      0     0|
%          |0         0       1        0       0      0     0|
% A  =     |0         0       0        1       0      0     0|
%          |0         0       0        0       1      0     0|
%          |0         0       0        0       0      1     0|
%          |0         0       0        0       0      0     1|
%
% R_k  =   |R_y       0   |
%          |0         R_e |
%
%                 |delta_k^3/3   delta_k^2/2       0        0 |
% R_y  = sigma_y  |delta_k^2/2       delta_k       0        0 |, % Kinematic of
%                 |0         0       delta_k^3/3   delta_k^2/2|  % the ellipsoid
%                 |0         0       delta_k^2/2       delta_k|
%
%          |sigma_Hx^2        0          0|
% R_e  =   |0        sigma_Hy^2          0|, % Kinematic of
%          |0         0      sigma_theta^2|  % the ellipsoid
%
%   - Color model
%       8x8x8 bins (deafult)
%       RGB colour space
%       Epachenikov kernel
%
% Example of usage:
%
% pf_colortracker('data\CAVIAR_Browse_WhileWaiting1.avi', 
%                 'data\CAVIAR_Browse_WhileWaiting1_1.txt',
%                 'data\CAVIAR_Browse_WhileWaiting1_1_Tracking_data.mat');
%
%
% Author:   Juan C. San Miguel (Video Processing and Understanding Lab)
% E-mail:   juancarlos.sanmiguel@uam.es
% URL:      http://www-vpu.ii.uam.es/~jcs
% Date:     June 2010
% Version:  1.0